﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class LevelBlockFactory : MonoBehaviour
{


    public LevelBlock CreateNewLevelBlock(Vector3 spawnPos, int blockSize)
    {
        LevelBlock lblock = new LevelBlock();

		GameObject parent = GameObject.CreatePrimitive(PrimitiveType.Cube);
		parent.transform.position = spawnPos;
		parent.GetComponent<MeshRenderer> ().enabled = false;
		lblock.setParent(parent);

        List<GameObject> obs = generateObstacles(blockSize, parent);
        lblock.setObs(obs);


		generateFirefiles(blockSize, parent);

		generateBackground(blockSize, parent);

        return lblock;
    }

    public LevelBlock CreateBlankLevelBlock(Vector3 spawnPos, int blockSize)
    {
        LevelBlock lblock = new LevelBlock();

		GameObject parent = GameObject.CreatePrimitive(PrimitiveType.Cube);
		parent.transform.position = spawnPos;
		lblock.setParent(parent);

        lblock.setObs(new List<GameObject>());
        
        generateBackground(blockSize, parent);

        return lblock;
    }


	private List<GameObject> generateObstacles(int blockSize, GameObject parent)
    {
        // get boundary width and height
        // choose random point and spawn object
        // later separate into ground / other objects

        int minDist = 10;

        List<GameObject> obsList = new List<GameObject>();



        int ran = Random.Range(1, 2);

        for (int i = 0; i < ran; i++)
        {
			int randomType = Random.Range (0, 10);
			if (randomType < 6) {
				Vector3 ranPos;
				if (obsList.Count == 0) {
					ranPos = RandomSkyPos (blockSize, parent);
				} else {
					bool goodPos = false;
					do {
						ranPos = RandomPos (blockSize, parent);
						for (int x = 0; x < obsList.Count; x++) {
							if (Vector3.Distance (ranPos, obsList [x].transform.position) > minDist) {
								goodPos = true;
							}
						}
					} while (!goodPos);
				}

				Object[] prefabs = Resources.LoadAll ("Obstacles");
				GameObject prefab = (GameObject) prefabs [Random.Range (0, prefabs.Length)];
				ranPos = new Vector3 (ranPos.x, prefab.transform.position.y, prefab.transform.position.z);
				GameObject obs = (GameObject) Instantiate (prefab, ranPos, prefab.transform.rotation);
				obs.transform.parent = parent.transform;
				obsList.Add (obs);
			} else {
				Vector3 ranPos;
				if (obsList.Count == 0) {
					ranPos = RandomGroundPos (blockSize, parent);
				} else {
					bool goodPos = false;
					do {
						ranPos = RandomGroundPos (blockSize, parent);
						for (int x = 0; x < obsList.Count; x++) {
							if (Vector3.Distance (ranPos, obsList [x].transform.position) > minDist) {
								goodPos = true;
							}
						}
					} while (!goodPos);
				}

				Object[] prefabs = Resources.LoadAll ("GroundObstacles");
				GameObject prefab = (GameObject) prefabs [Random.Range (0, prefabs.Length)];
				ranPos = new Vector3 (ranPos.x, prefab.transform.position.y, prefab.transform.position.z);
				GameObject obs = (GameObject) Instantiate (prefab, ranPos, prefab.transform.rotation);
				obs.transform.parent = parent.transform;
				obsList.Add (obs);
			}
        }
        return obsList;
    }


	private List<GameObject> generateFirefiles(int blockSize, GameObject parent)
    {

        List<GameObject> ffList = new List<GameObject>();
		int numberOfFireFlies = Random.Range(0, 3);

        for (int i = 0; i < numberOfFireFlies; i++)
        {
			int typeOfFireFly = Random.Range (0, 10);
			Vector3 randomPos;

			// 20% chance
			if (typeOfFireFly < 2) {
				randomPos = RandomPos (blockSize, parent);
				GameObject obj = (GameObject) Resources.Load ("Fireflies/FireflyOrange");
				GameObject ff = (GameObject)Instantiate (obj, randomPos, obj.transform.rotation);
				ff.transform.parent = parent.transform;
				ffList.Add (ff);
			}

			// 30% chance
			if (typeOfFireFly >= 2 && typeOfFireFly <= 4 ) {
				randomPos = RandomPos (blockSize, parent);
				GameObject obj = (GameObject) Resources.Load ("Fireflies/FireflyPink");
				GameObject ff = (GameObject)Instantiate (obj, randomPos, obj.transform.rotation);
				ff.transform.parent = parent.transform;
				ffList.Add (ff);
			}

			// 50% chance
			if (typeOfFireFly > 4 ) {
				randomPos = RandomPos (blockSize, parent);
				GameObject obj = (GameObject) Resources.Load ("Fireflies/FireflyYellow");
				GameObject ff = (GameObject)Instantiate (obj, randomPos, obj.transform.rotation);
				ff.transform.parent = parent.transform;
				ffList.Add (ff);
			}
        }
        return ffList;
    }


	private List<GameObject> generateBackground(int blockSize, GameObject parent)
    {
        return new List<GameObject>();
    }

    private Vector3 RandomPos(int blockSize, GameObject parent)
    {
        return new Vector3(Random.Range(-blockSize / 2, blockSize / 2) + parent.transform.position.x, Random.Range(-blockSize / 2, blockSize / 2) + parent.transform.position.y, 0);
    }

	private Vector3 RandomSkyPos(int blockSize, GameObject parent)
	{
		return new Vector3(Random.Range(-blockSize / 2, blockSize / 2) + parent.transform.position.x, 5 + parent.transform.position.y, 0);
	}

	private Vector3 RandomGroundPos(int blockSize, GameObject parent)
	{
		return new Vector3(Random.Range(-blockSize / 2, blockSize / 2) + parent.transform.position.x, -blockSize / 2, 0);
	}
}
